# -*- coding: utf-8 -*-
#
#  ConfigManager
#
# Coded by dre (c) 2014 - 2021
# Support: www.dreamboxtools.de
# E-Mail: dre@dreamboxtools.de
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.

from __future__ import print_function
from Components.config import config, ConfigSubsection, ConfigSelection, ConfigYesNo, configfile, getConfigListEntry
from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Components.Pixmap import Pixmap
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.Standby import TryQuitMainloop
from Tools.Directories import fileExists, resolveFilename, SCOPE_CURRENT_SKIN
from Tools.LoadPixmap import LoadPixmap
from enigma import RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_VALIGN_CENTER, eListboxPythonMultiContent, gFont, eConsoleAppContainer, getDesktop
from skin import TemplatedListFonts, componentSizes
import os, re

# for localized messages
from . import _

config.plugins.configmanager = ConfigSubsection()
config.plugins.configmanager.backuppath = ConfigSelection([("/media/hdd", "/media/hdd"), ("/media/usb", "/media/usb"), ("/media/cf", "/media/cf")], default = "/media/hdd")
config.plugins.configmanager.rememberselection = ConfigYesNo(default=False)

def main(session,**kwargs):
	session.open(ConfigViewerScreen)

class ConfigMenuList(MenuList):
	SKIN_COMPONENT_KEY = "ConfigManagerList"
	SKIN_COMPONENT_ICON_HEIGHT = "iconHeight"
	SKIN_COMPONENT_ICON_WIDTH = "iconWidth"
	SKIN_COMPONENT_ITEM_HEIGHT = "itemHeight"
	SKIN_COMPONENT_ITEM_WIDTH = "itemWidth"
	
	
	def __init__(self, list):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		
		isFHD = False
		sz_w = getDesktop(0).size().width()
		if sz_w >= 1920:
			isFHD = True

		sizes = componentSizes[ConfigMenuList.SKIN_COMPONENT_KEY]
		self.componentItemHeight = sizes.get(ConfigMenuList.SKIN_COMPONENT_ITEM_HEIGHT, 40 if isFHD else 25)
		self.componentItemWidth = sizes.get(ConfigMenuList.SKIN_COMPONENT_ITEM_WIDTH, 1480 if isFHD else 695)
		self.iconWidth = sizes.get(ConfigMenuList.SKIN_COMPONENT_ICON_WIDTH, 40 if isFHD else 25)
		self.iconHeight = sizes.get(ConfigMenuList.SKIN_COMPONENT_ICON_HEIGHT, 40 if isFHD else 25)
		
		self.l.setItemHeight(self.componentItemHeight)
		tlf = TemplatedListFonts()
		self.l.setFont(0, gFont(tlf.face(tlf.SMALL), tlf.size(tlf.SMALL)))
		self.l.setBuildFunc(self.buildEntry)
		
	def buildEntry(self, entry, value, selected = False):
		isFHD = False
		sz_w = getDesktop(0).size().width()
		if sz_w >= 1920:
			isFHD = True
		
		res = [ (entry, value, selected) ]
		if selected is False:
			icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_off.png"))
		else:
			icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_on.png"))
		res.append(MultiContentEntryPixmapAlphaTest(pos=(0,0), size=(self.iconWidth,self.iconHeight), png = icon))
		res.append(MultiContentEntryText(pos=(self.iconWidth,0),size=(self.componentItemWidth, self.componentItemHeight), font=0, flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, text=entry))
		res.append(MultiContentEntryText(text=value))
		return res

class CMPluginConfigScreen(Screen, ConfigListScreen, HelpableScreen):
	sz_w = getDesktop(0).size().width()
	if sz_w >= 1920:
		skin = """
		<screen position="center,center" size="800,250" title="Plugin Config" >
			<ePixmap pixmap="skin_default/buttons/green.png" position="10,5" size="300,65" alphatest="on"/>
	    	<widget name="SaveText" position="10,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
	    	<eLabel	position="10,70" size="780,1" backgroundColor="grey"/>
	    	<widget name="config" position="10,80" enableWrapAround="1" size="780,150" alternate_row_color="#00333333" />
		</screen>"""	
	else:
		skin = """
		<screen position="center,center" size="500,140" title="Plugin Config" >
			<ePixmap pixmap="skin_default/buttons/green.png" position="10,5" size="360,40" alphatest="on" />
	    	<widget name="SaveText" position="10,5" size="360,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
	    	<eLabel	position="10,50" size="480,1" backgroundColor="grey"/>
	    	<widget name="config" position="10,60" enableWrapAround="1" size="480,60" alternate_row_color="#00333333" />
		</screen>"""		
		
	def __init__(self, session, args = 0):
		self.skin = CMPluginConfigScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.list = []
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, _("Close")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveConfig, _("Save config")),
		}, -1)
		
		self["SaveText"] = Label(_("Save config and close"))
		
		ConfigListScreen.__init__(self, self.list)
		
		self.list.append(getConfigListEntry(_("Back up path:"), config.plugins.configmanager.backuppath))
		self.list.append(getConfigListEntry(_("Save last selection to config:"), config.plugins.configmanager.rememberselection))
		self["config"].setList(self.list)
		
	def saveConfig(self):
		for x in self.list:
			x[1].save()
		self.close()

class CleanUpWizardScreen(Screen, HelpableScreen):
	sz_w = getDesktop(0).size().width()
	if sz_w >= 1920:
		skin = """
		<screen position="center,90" size="1500,900" title="Clean Up Wizard" >
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="300,65" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
	    	<eLabel	position="10,80" size="1480,1" backgroundColor="grey"/>
			<widget name="FoundConfigDescr" position="10,85" size="490,40" halign="left" font="Regular;30" />
			<widget name="FoundConfig" position="500,85" size="100,40" halign="left" font="Regular;30" />
			<widget name="LeftBehindDescr" position="10,130" size="490,40" halign="left" font="Regular;30" />
			<widget name="LeftBehind" position="500,130" size="100,40" halign="left" font="Regular;30" />	
			<eLabel	position="10,175" size="1480,1" backgroundColor="grey"/>
	    	<widget name="menulist" position="10,180" enableWrapAround="1" size="1480,680" scrollbar="showOnDemand" alternate_row_color="#00333333" />
		</screen>"""
	else:
		skin = """
		<screen position="center,120" size="620,520" title="Clean Up Wizard" >
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="360,40" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="360,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<eLabel	position="10,50" size="600,1" backgroundColor="grey"/>
			<widget name="FoundConfigDescr" position="10,57" size="480,25" halign="left" font="Regular;18" />
			<widget name="FoundConfig" position="500,57" size="100,25" halign="left" font="Regular;18" />
			<widget name="LeftBehindDescr" position="10,83" size="480,25" halign="left" font="Regular;18" />
			<widget name="LeftBehind" position="500,83" size="100,25" halign="left" font="Regular;18" />	
			<eLabel	position="10,110" size="600,1" backgroundColor="grey"/>
	    	<widget name="menulist" position="10,120" enableWrapAround="1" size="600,390" alternate_row_color="#00333333"/>
		</screen>"""
		
	def __init__(self, session, pluginConfigEntries):
		self.skin = CleanUpWizardScreen.skin
		self.session = session
		self.pluginConfigEntries = pluginConfigEntries
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.selectForRemoval, _("Select entry for removal")),
			"cancel":	(self.close, _("Close screen")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeSelectedEntries, _("Remove selected entries")),
		}, -1)
		
		self.menulist = []
		self.pluginConfigList = []
		self.pluginConfigDict = {}
		self["menulist"] = ConfigMenuList([])
		
		self.selectedEntries = []
		
		self["FoundConfig"] = Label()
		self["FoundConfigDescr"] = Label(_("Config entries found:"))
		
		self["LeftBehind"] = Label()
		self["LeftBehindDescr"] = Label(_("Config entries left behind:"))
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Remove selected entries"))
		
		self.foundConfig = "0"
		self.leftBehind = "0"
		
		self["FoundConfig"].setText(self.foundConfig)
		self["LeftBehind"].setText(self.leftBehind)
		
		self.dirlist = ["/usr/lib/enigma2/python/Plugins/Extensions", "/usr/lib/enigma2/python/Plugins/SystemPlugins"]
		
		self.subdirlist = []
		self.subdirlist = self.getSubDirectories(self.dirlist)
		
		for i in range(len(self.subdirlist)):
			for p in range(len(self.subdirlist[i])):
				path = self.dirlist[i] + "/" + self.subdirlist[i][p]
				pyfiles = self.getpyFiles(path)
				for j in range(len(pyfiles)):
					self.content = ""
					checkFile = open(path + "/" + pyfiles[j], "r")
					self.content = checkFile.read()
					checkFile.close()
					configentries = re.findall("config\.(.*?)\s*={1}\s*([a-zA-z_0-9]+)(\((.*?)\)|\((.*?)\[\s*$|\(\s*$)", self.content, re.MULTILINE)
					if len(configentries) > 0:
						for k in range(len(configentries)):
							# this is the safety net because there's so many possible ways to define configs
							if configentries[k][0].find(".value") == -1 and configentries[k][0].find(".addNotifier") == -1 and configentries[k][0].find("getValue") == -1 and configentries[k][0].find(".save()") == -1 and configentries[k][0].find(".index >") == -1:
								if configentries[k][1] == "ConfigSubsection":
									continue
								self.pluginConfigList.append("config." + configentries[k][0].strip())
								self.pluginConfigDict["config." + configentries[k][0].strip()] = configentries[k]

		self.foundConfig = len(self.pluginConfigList)
		self["FoundConfig"].setText(str(self.foundConfig))
		
		# pluginConfigEntries = /etc/enigma2/settings
		for c in self.pluginConfigEntries:
			# pluginConfigList = Sourcecode
			if c not in self.pluginConfigList:
				sublistFound = False
				sublistcheck = ""
				sublistcheck2 = re.findall("config\.plugins\.(.*?)\.(.*?)\.([0-9]+)\.(.*?)$", c)
				#a sublistcheck length greater than 0 means we have a ConfigSubList
				if len(sublistcheck2) > 0:
					sublistFound = False
					configDetails = self.pluginConfigDict.get("config.plugins.%s.%s" %(sublistcheck2[0][0], sublistcheck2[0][1]), None)
					if configDetails is not None:
						if configDetails[1] == "ConfigSubList" or configDetails[1] == "ConfigSubDict":
							sublistFound = True
				
				else:
					sublistcheck = re.findall("config\.plugins\.(.*?)\.(.*?)\.([0-9]+)$", c)
					if len(sublistcheck) > 0:
						sublistFound = False
						configDetails = self.pluginConfigDict.get("config.plugins.%s.%s" %(sublistcheck[0][0], sublistcheck[0][1]), None)
						if configDetails is not None:
							if configDetails[1] == "ConfigSubList" or configDetails[1] == "ConfigSubDict":
								sublistFound = True
					else:
						sublistcheck3 = re.findall("config\.plugins\.(.*?)\.(.*?)\.(.*?)$", c)
						if len(sublistcheck3) > 0:
							sublistFound = False
							configDetails = self.pluginConfigDict.get("config.plugins.%s.%s" %(sublistcheck3[0][0], sublistcheck3[0][1]), None)
							if configDetails is not None:
								if configDetails[1] == "ConfigSubList" or configDetails[1] == "ConfigSubDict":
									sublistFound = True
				if sublistFound == False:
					self.menulist.append((c, None, False))
		self.differenceList = []
		self.differenceList = self.menulist
		self["menulist"].l.setList(self.menulist)
		
		self.leftBehind = len(self.menulist)
		self["LeftBehind"].setText(str(self.leftBehind))
		if self.leftBehind == 0:
			self["ButtonRed"].hide()
			self["ButtonRedText"].hide()
		
	def getSubDirectories(self, dir):
		return [ [subdirectory for subdirectory in os.listdir(directory) if os.path.isdir(os.path.join(directory,subdirectory))] for directory in dir ]

	def getpyFiles(self, path):
		return [ pyfile for pyfile in os.listdir(path)
					if os.path.isfile(os.path.join(path, pyfile)) and pyfile.endswith(".py")]
					
	def removeSelectedEntries(self):
		if fileExists("/etc/enigma2/settings"):
			try:
				origFile = open('/etc/enigma2/settings', 'r')
				self.settingsList = []
				self.settingsList = origFile.readlines()
				
				for i in range(len(self.selectedEntries)):
					for k in range(len(self.settingsList)):
						entrysplit = self.settingsList[k].split('=')
						if entrysplit[0] == self.selectedEntries[i]:
							#a dirty hack
							self.settingsList[k] = "#"
							break
				
				origFile.close()
				
				os.system("touch /etc/enigma2/settings.new")
				os.system("chmod 644 /etc/enigma2/setttings.new")
				configFile = open('/etc/enigma2/settings.new', 'w')	
				for j in range(len(self.settingsList)):
					if self.settingsList[j] != "#":
						configFile.write(self.settingsList[j])
				configFile.close()

				self.doRestart()			
				
			except IOError:
				print("couldn't read settings")
				
	def selectForRemoval(self):
		if len(self.menulist) > 0:
			selectedMenuEntry = self["menulist"].getSelectedIndex()
			if self.menulist[selectedMenuEntry][2] == False:
				self.selectedEntries.append(self.menulist[selectedMenuEntry][0])
			else:
				for k in range(len(self.selectedEntries)):
					if self.selectedEntries[k] == self.menulist[selectedMenuEntry][0]:
						del self.selectedEntries[k]
						break
			self.updateList(jumpToFirst=False)
		
	def updateList(self, jumpToFirst=True):
		self.menulist = []
		self.alreadyAdded = False
	
		for i in range(len(self.differenceList)):
			self.markSelectedEntries(i)

		self["menulist"].l.setList(self.menulist)
		if jumpToFirst:
			self["menulist"].moveToIndex(0)
			
	def markSelectedEntries(self, index):
		self.index = index
		self.alreadyAdded = False
		for j in range(len(self.selectedEntries)):
			if self.differenceList[self.index][0] == self.selectedEntries[j]:
				self.menulist.append((self.differenceList[self.index][0], None, True))
				self.alreadyAdded = True
				break
		if self.alreadyAdded == False:
			self.menulist.append((self.differenceList[self.index][0], None, False))
		else:
			self.alreadyAdded = False
			
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)			

class RestoreSelectionScreen(Screen):
	sz_w = getDesktop(0).size().width()
	if sz_w >= 1920:
		skin = """
		<screen position="center,center" size="800,400" title="Config Manager" >
			<widget name="restoreselectionlist" position="10,10" enableWrapAround="1" size="780,380" scrollbarMode="showOnDemand"  alternate_row_color="#00333333" />
		</screen>"""
	else:
		skin = """
		<screen position="center,center" size="520,80" title="Config Manager" >
			<widget name="restoreselectionlist" position="10,10" enableWrapAround="1" size="500,60" scrollbarMode="showOnDemand"  alternate_row_color="#00333333" />
		</screen>"""			
	
	def __init__(self, session):
		self.skin = RestoreSelectionScreen.skin
		self.session = session
		Screen.__init__(self, session)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.runCommand, _("Run selected task")),
			"cancel":	(self.close, _("Close screen")),
		}, -1)
		
		self.entries = []
		self.backupdir = ""
		self.backupdir = config.plugins.configmanager.backuppath.value
		if fileExists("%s/settings.tar.gz" %(self.backupdir)):
			self.entries.append( (_("Restore settings"), 0))
			self.entries.append( (_("Merge back up into settings"), 1))

		self["restoreselectionlist"] = MenuList(self.entries)
		
	def runCommand(self):
		menuitem = self["restoreselectionlist"].getCurrent()[1]
		#restore settings
		if menuitem == 0:
			self.result = os.system("tar -xzf %s/settings.tar.gz -C /etc/enigma2/" %(self.backupdir))		
			if (int(self.result)/256)==0:
				self.doRestart()
		#merge back up	
		elif menuitem == 1:
			self.result = os.system("tar -xzf %s/settings.tar.gz -C /tmp/" %(self.backupdir))
			if (int(self.result)/256)==0:
				self.doMerge()
		
	def doMerge(self):
		self.writeFile = False
		if fileExists("/etc/enigma2/settings") and fileExists("/tmp/settings.new"):
			try:
				buf = []
				backUpFile = open('/tmp/settings.new', 'r')
				mergeList = backUpFile.readlines()
				configFile = open('/etc/enigma2/settings', 'r')
				self.writeFile = True

				for line in configFile:
					for l in range(len(mergeList)):
						if mergeList[l] != '#' and mergeList[l] != '\n':
							backupsplit = mergeList[l].split('=')
							if line.startswith(backupsplit[0]):
								line = line.replace(line, mergeList[l])
								#kinda remove the line to be able to add all lines that are not in configFile
								mergeList[l] = "#"
								break
					buf.append(line)
					
				#this is needed to add lines that are not present in configFile
				for k in range(len(mergeList)):
					if mergeList[k] != '#' and mergeList[k] != '\n':
						buf.append(mergeList[k])
								
			except IOError:
				print("couldn't read settings")
			backUpFile.close()
			configFile.close()
			
			if self.writeFile:
				os.system("touch /etc/enigma2/settings.new")
				os.system("chmod 644 /etc/enigma2/setttings.new")
				configFile = open('/etc/enigma2/settings.new', 'w')	
				for line in buf:
					configFile.write(line)
				configFile.close()
		
				self.doRestart()
	
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)				

class ConfigBackUpScreen(Screen,HelpableScreen):
	sz_w = getDesktop(0).size().width()
	if sz_w >= 1920:
		skin = """
		<screen position="center,center" size="930,780" title="Config Manager - Back Up">
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="300,60" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="300,60" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<eLabel	position="10,70" size="910,1" backgroundColor="grey"/>
			<widget name="progresstext" position="10,75" size="910,40" halign="left" font="Regular;27" />	
			<eLabel	position="10,120" size="910,1" backgroundColor="grey"/>
	    	<widget name="backedUp" position="10,130" enableWrapAround="1" size="910,630" alternate_row_color="#00333333" />
		</screen>"""
	else:		
		skin = """
		<screen position="center,center" size="620,520" title="Config Manager - Back Up">
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="200,40" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<eLabel	position="10,50" size="600,1" backgroundColor="grey"/>
			<widget name="progresstext" position="10,55" size="600,25" halign="left" font="Regular;18" />	
			<eLabel	position="10,80" size="600,1" backgroundColor="grey"/>
	    	<widget name="backedUp" position="10,90" enableWrapAround="1" size="600,420" alternate_row_color="#00333333" />
		</screen>"""
	
	def __init__(self, session, selection):
		self.skin = ConfigBackUpScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.closeAndClean, _("Close screen")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.close, _("Close screen")),
		}, -1)
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Close"))	
		
		self.selection = selection
		self.backedUpList = []
		self["backedUp"] = MenuList(self.backedUpList)
		
		total = len(self.selection)
		
		self["progresstext"] = Label(_("0/%s backed up") %(total))
		
		counter = 0
		if fileExists("/etc/enigma2/settings"):
			try:
				configFile = open('/etc/enigma2/settings', 'r')
				for i in range(len(self.selection)):
					for line in configFile:
						if line.startswith(self.selection[i]):
							counter = counter+1
							self["progresstext"].setText(_("%s/%s backed up") %(counter, total))
							self.backedUpList.append(line)
							self["backedUp"].l.setList(self.backedUpList)
							break
				os.system("touch /tmp/settings.new")
				os.system("chmod 644 /tmp/setttings.new")
				
				backUpFile = open('/tmp/settings.new', 'w')	
				for k in range(len(self.backedUpList)):
					backUpFile.write(self.backedUpList[k])
				backUpFile.close()
				
				self.backupdir = ""
				self.backupdir = config.plugins.configmanager.backuppath.value
				os.system("tar -zcf %s/settings.tar.gz -C /tmp settings.new" %(self.backupdir))
				
			except IOError:
				print("couldn't read settings")
			configFile.close()
			
	def closeAndClean(self):
		os.system("rm -f /tmp/settings.new")
		self.close()
		
class ConfigViewerScreen(Screen,HelpableScreen):
	sz_w = getDesktop(0).size().width()
	if sz_w >= 1920:
		skin = """
		<screen position="center,90" size="1500,900" title="Config Manager" >
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="300,65" alphatest="on" />
    		<widget name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="310,5" size="300,65" alphatest="on" />
	    	<widget name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="610,5" size="300,65" alphatest="on" />
	     	<widget name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="910,5" size="300,65" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
	    	<widget name="ButtonGreenText" position="310,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
    		<widget name="ButtonYellowText" position="610,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
	    	<widget name="ButtonBlueText" position="910,5" size="300,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
	    	<eLabel position="10,75" size="1480,1" backgroundColor="grey" />
	    	<widget name="menulist" position="10,80" size="1480,600" enableWrapAround="1" scrollbarMode="showOnDemand"  alternate_row_color="#00333333" />
	    	<eLabel position="10,690" size="1480,1" backgroundColor="grey" />
	    	<eLabel position="750,695" size="1,195" backgroundColor="grey" />
	    	<widget name="entryinfo" position="10,710" size="540,150" font="Regular;30" halign="center" valign="center" />
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_0.png" position="760,720" size="40,20"/>
			<widget name="allEntries" font="Regular;18" position="820,720" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_1.png" position="760,750" size="40,20"/>
			<widget name="plugEntries" font="Regular;18" position="820,750" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_2.png" position="760,780" size="40,20"/>
			<widget name="enigmaEntries" font="Regular;18" position="820,780" size="200,25" transparent="1"/>
			<widget name="key3" pixmap="skin_default/buttons/key_3.png" position="760,810" size="40,20" alphatest="on"/>
			<widget name="merlinEntries" font="Regular;18" position="820,810" size="200,25" transparent="1"/>
			<widget name="selectAll" font="Regular;18" position="820,840" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_7.png" position="760,840" size="40,20"/>	
			<widget name="deselectAll" font="Regular;18" position="820,870" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_9.png" position="760,870" size="40,20"/>						
		</screen>"""
	else:
		skin = """
		<screen position="center,90" size="820,600" title="Config Manager" >
		    <widget name="ButtonRed" pixmap="skin_default/buttons/red.png" position="10,5" size="200,65" alphatest="on" />
    		<widget name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="210,5" size="200,65" alphatest="on" />
	    	<widget name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="410,5" size="200,65" alphatest="on" />
	     	<widget name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="610,5" size="200,65" alphatest="on" />
	    	<widget name="ButtonRedText" position="10,5" size="200,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<widget name="ButtonGreenText" position="210,5" size="200,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
    		<widget name="ButtonYellowText" position="410,5" size="200,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<widget name="ButtonBlueText" position="610,5" size="200,65" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#18188b" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
	    	<eLabel position="10,75" size="800,1" backgroundColor="grey" />
	    	<widget name="menulist" position="10,80" size="800,325" enableWrapAround="1" scrollbarMode="showOnDemand"  alternate_row_color="#00333333" />
	    	<eLabel position="10,410" size="800,1" backgroundColor="grey" />
	    	<eLabel position="550,415" size="1,180" backgroundColor="grey" />
	    	<widget name="entryinfo" position="10,415" size="540,120" font="Regular;20" halign="center" valign="center" />
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_0.png" position="560,420" size="40,20"/>
			<widget name="allEntries" font="Regular;18" position="620,420" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_1.png" position="560,450" size="40,20"/>
			<widget name="plugEntries" font="Regular;18" position="620,450" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_2.png" position="560,480" size="40,20"/>
			<widget name="enigmaEntries" font="Regular;18" position="620,480" size="200,25" transparent="1"/>
			<widget name="key3" pixmap="skin_default/buttons/key_3.png" position="560,510" size="35,25" alphatest="on"/>
			<!--ePixmap alphatest="on" pixmap="skin_default/buttons/key_3.png" position="560,510" size="40,20"/-->
			<widget name="merlinEntries" font="Regular;18" position="620,510" size="200,25" transparent="1"/>
			<widget name="selectAll" font="Regular;18" position="620,540" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_7.png" position="560,540" size="40,20"/>	
			<widget name="deselectAll" font="Regular;18" position="620,570" size="200,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_9.png" position="560,570" size="40,20"/>						
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = ConfigViewerScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self["NumberActions"] = HelpableActionMap(self, "NumberActions",
		{
			"0":		(self.updateList, _("Show complete config")),
			"1":		(self.showPluginConfigOnly, _("Show plugin config only")),
			"2":		(self.showEnigmaConfigOnly, _("Show Enigma2 config only")),
			"3":		(self.showMerlinConfigOnly, _("Show Merlin config only")),
			"7":		(self.selectAll, _("Select all entries")),
			"9":		(self.deselectAll, _("Deselect all entries")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeEntries, _("Delete selected entries")),
			"green":	(self.startBackUp, _("Back up selected entries")),
			"yellow":	(self.showRestoreSelection, _("Restore from back up")),
			"blue":		(self.runCleanUpWizard, _("Run clean up wizard")),
		}, -1)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.selectForBackUp, _("Select entry for back up")),
			"cancel":	(self.closePlugin, _("Close plugin")),
		}, -1)
		
		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":		(self.openPluginConfig, _("Open plugin config screen")),
		}, -1)

		self["allEntries"] = Label(_("All entries"))
		self["plugEntries"] = Label(_("Plugin config only"))
		self["enigmaEntries"] = Label(_("Enigma2 config only"))
		self["merlinEntries"] = Label(_("Merlin config only"))
		self["selectAll"] = Label(_("Select all entries"))
		self["deselectAll"] = Label(_("Deselect all entries"))
		self["key3"] = Pixmap()
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Delete selected entries"))		
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreenText"] = Label(_("Back up selected entries"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowText"] = Label(_("Restore from back up"))
		self["ButtonBlue"] = Pixmap()
		self["ButtonBlueText"] = Label(_("Run clean up wizard"))
		
		self["ButtonYellow"].hide()
		self["ButtonYellowText"].hide()
		self["merlinEntries"].hide()
		self["key3"].hide()
		
		self.setYellowButton()
		
		self.hasMerlinEntries = False
		
		self.menulist = []
		self.pluginConfigEntries = []
		self["menulist"] = ConfigMenuList([])
		
		self.selectedEntries = []
		if config.plugins.configmanager.rememberselection.value:
			if fileExists("/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection"):
				try:
					selectionFile = open('/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection', 'r')
					selbuf = selectionFile.readline().strip("\n[]").split(",")
					selectionFile.close()
					for k in range(len(selbuf)):
						self.selectedEntries.append(selbuf[k].strip())
				
				except IOError:
					print("couldn't read settings")
		
		self["entryinfo"] = Label()
		
		self.currentView = "ALL"
		
		self.createList()
		
		self["menulist"].onSelectionChanged.append(self.updateEntryinfo)
	
	def openPluginConfig(self):
		self.session.open(CMPluginConfigScreen)
	
	def updateEntryinfo(self):
		selectedMenuEntry = self["menulist"].getSelectedIndex()
		self["entryinfo"].setText(self.menulist[selectedMenuEntry][1])
	
	def readConfig(self):
		if fileExists("/etc/enigma2/settings"):
			try:
				configFile = open('/etc/enigma2/settings', 'r')
				lines = []
				for line in configFile:
					if line[0] != '#' and line != '\n':
						val = line.strip().split('=')
						lines.append([val[0],val[1]])
				return lines
			
			except IOError:
				print("couldn't read settings")
				return None

			configFile.close()

	def createList(self):
		self.data = self.readConfig()
		
		for i in range(len(self.data)):
			if self.data[i][0].startswith("config.plugins"):
				self.data[i].append("PLUGIN")
				self.pluginConfigEntries.append(self.data[i][0])
			elif self.data[i][0].startswith("config.merlin"):
				self.data[i].append("MERLIN")
				self.hasMerlinEntries = True
			else:
				self.data[i].append("ENIGMA")

			self.markSelectedEntries(i)
		
		self["menulist"].l.setList(self.menulist)
		
		if self.hasMerlinEntries:
			self.setKey3()		

		self.updateEntryinfo()

	def markSelectedEntries(self, index):
		self.index = index
		self.alreadyAdded = False
		for j in range(len(self.selectedEntries)):
			if self.data[self.index][0] == self.selectedEntries[j]:
				#self.menulist.append(ConfigMenuListEntry(self.data[self.index][0], self.data[self.index][1], selected = True))
				self.menulist.append((self.data[self.index][0], self.data[self.index][1], True))
				self.alreadyAdded = True
				break
		if self.alreadyAdded == False:
			#self.menulist.append(ConfigMenuListEntry(self.data[self.index][0], self.data[self.index][1]))
			self.menulist.append((self.data[self.index][0], self.data[self.index][1], False))
		else:
			self.alreadyAdded = False
			
	def updateList(self, configtype="ALL", jumpToFirst=True):
		self.menulist = []
		self.currentView = configtype
		self.alreadyAdded = False
	
		if self.currentView == "ALL":
			for i in range(len(self.data)):
				self.markSelectedEntries(i)
		else:
			for i in range(len(self.data)):
				if self.data[i][2] == self.currentView:
					self.markSelectedEntries(i)
		
		self["menulist"].l.setList(self.menulist)
		if jumpToFirst:
			self["menulist"].moveToIndex(0)
		self.updateEntryinfo()
	
	def showPluginConfigOnly(self):
		self.updateList("PLUGIN")
	
	def showMerlinConfigOnly(self):
		if self.hasMerlinEntries:
			self.updateList("MERLIN")
		
	def showEnigmaConfigOnly(self):
		self.updateList("ENIGMA")
		
	def selectAll(self):
		for k in range(len(self.menulist)):
			if self.menulist[k][2] == False:
				self.selectedEntries.append(self.menulist[k][0])
				
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()
			
		self.updateList(self.currentView, jumpToFirst=False)
				
	def deselectAll(self):
		self.selectedEntries = []
				
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()
			
		self.updateList(self.currentView, jumpToFirst=False)

	def selectForBackUp(self):
		selectedMenuEntry = self["menulist"].getSelectedIndex()
		if self.menulist[selectedMenuEntry][2] == False:
			self.selectedEntries.append(self.menulist[selectedMenuEntry][0])
		else:
			for k in range(len(self.selectedEntries)):
				if self.selectedEntries[k] == self.menulist[selectedMenuEntry][0]:
					del self.selectedEntries[k]
					break
		#save it to config
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()

		self.updateList(self.currentView, jumpToFirst=False)

	def saveCurrentSelection(self):
		if not fileExists("/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection"):
			os.system("touch /usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection")
		
		os.system("echo %s > /usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection" %(self.selectedEntries))
		
	def startBackUp(self):
		self.session.openWithCallback(self.setYellowButton, ConfigBackUpScreen, self.selectedEntries)
	
	def setYellowButton(self):
		self.backupdir = ""
		self.backupdir = config.plugins.configmanager.backuppath.value
		if fileExists("%s/settings.tar.gz" %(self.backupdir)):
			self["ButtonYellow"].show()
			self["ButtonYellowText"].show()
			
	def setKey3(self):
		self["merlinEntries"].show()
		self["key3"].show()
	
	def removeEntries(self):
		confirmbox = self.session.openWithCallback(self.removeEntriesConfirmed,MessageBox,_("%d entries will be removed. Proceed?") %(len(self.selectedEntries)), MessageBox.TYPE_YESNO)
		confirmbox.setTitle(_("Config Manager"))
		
	def removeEntriesConfirmed(self, answer):
		if answer is True:
			configfile.save()
			if fileExists("/etc/enigma2/settings"):
				try:
					buf = []
					origFile = open('/etc/enigma2/settings', 'r')
					self.settingsList = []
					self.settingsList = origFile.readlines()
				
					for i in range(len(self.selectedEntries)):
						for k in range(len(self.settingsList)):
							entrysplit = self.settingsList[k].split('=')
							if entrysplit[0] == self.selectedEntries[i]:
								#a dirty hack
								self.settingsList[k] = "#"
								break
				
					origFile.close()
				
					os.system("touch /etc/enigma2/settings.new")
					os.system("chmod 644 /etc/enigma2/setttings.new")
					configFile = open('/etc/enigma2/settings.new', 'w')	
					for j in range(len(self.settingsList)):
						if self.settingsList[j] != "#":
							configFile.write(self.settingsList[j])
					configFile.close()

					self.doRestart()			
				
				except IOError:
					print("couldn't read settings")
			
	def runCleanUpWizard(self):
		self.session.open(CleanUpWizardScreen, self.pluginConfigEntries)
		
	def showRestoreSelection(self):
		self.session.open(RestoreSelectionScreen)
		
	def closePlugin(self):
		self.close()
		
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings.\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)
		
def Plugins(**kwargs):
    return [PluginDescriptor(name="Config Manager", description=_("Plugin to view/edit settings"), where = [PluginDescriptor.WHERE_PLUGINMENU], fnc=main, icon = "configmanager.png")]
